@@ -1,8 +1,9 @@
-from __future__ import absolute_import
+from __future__ import absolute_import, division
 import os
 import traceback
 import sys
 
+import math
 import urwid
 
 from netlib import odict
@@ -24,8 +25,9 @@ def _mkhelp():
         ("A", "accept all intercepted flows"),
         ("a", "accept this intercepted flow"),
         ("b", "save request/response body"),
-        ("d", "delete flow"),
         ("D", "duplicate flow"),
+        ("d", "delete flow"),
+        ("E", "export"),
         ("e", "edit request/response"),
         ("f", "load full body data"),
         ("m", "change body display mode for this entity"),
@@ -94,13 +96,13 @@ footer = [
 
 
 class FlowViewHeader(urwid.WidgetWrap):
+
     def __init__(self, master, f):
         self.master, self.flow = master, f
         self._w = common.format_flow(
             f,
             False,
             extended=True,
-            padding=0,
             hostheader=self.master.showhost
         )
         signals.flow_change.connect(self.sig_flow_change)
@@ -111,7 +113,6 @@ class FlowViewHeader(urwid.WidgetWrap):
                 flow,
                 False,
                 extended=True,
-                padding=0,
                 hostheader=self.master.showhost
             )
 
@@ -207,10 +208,26 @@ class FlowView(tabs.Tabs):
         if description == "No content" and isinstance(message, HTTPRequest):
             description = "No request content (press tab to view response)"
 
+        # If the users has a wide terminal, he gets fewer lines; this should not be an issue.
+        chars_per_line = 80
+        max_chars = max_lines * chars_per_line
+        total_chars = 0
         text_objects = []
         for line in lines:
-            text_objects.append(urwid.Text(line))
-            if len(text_objects) == max_lines:
+            txt = []
+            for (style, text) in line:
+                if total_chars + len(text) > max_chars:
+                    text = text[:max_chars - total_chars]
+                txt.append((style, text))
+                total_chars += len(text)
+                if total_chars == max_chars:
+                    break
+
+            # round up to the next line.
+            total_chars = int(math.ceil(total_chars / chars_per_line) * chars_per_line)
+
+            text_objects.append(urwid.Text(txt))
+            if total_chars == max_chars:
                 text_objects.append(urwid.Text([
                     ("highlight", "Stopped displaying data after %d lines. Press " % max_lines),
                     ("key", "f"),
@@ -557,6 +574,18 @@ class FlowView(tabs.Tabs):
                 callback = self.master.save_one_flow,
                 args = (self.flow,)
             )
+        elif key == "E":
+            signals.status_prompt_onekey.send(
+                self,
+                prompt = "Export",
+                keys = (
+                    ("as curl command", "c"),
+                    ("as python code", "p"),
+                    ("as raw request", "r"),
+                ),
+                callback = common.export_prompt,
+                args = (self.flow,)
+            )
         elif key == "|":
             signals.status_prompt_path.send(
                 prompt = "Send flow to script",
